/*
 *  ps_main.h
 *  ProteinSplitter
 *
 *  Created by Naoto Morikawa on 7/21/06.
 *  Copyright (C) 2006 Naoto Morikawa. All rights reserved.
 *
 *  2006-10-01 version 0.1 release
 */

#define STDOUT stdout
#define STDERR stdout
#define LINE_WIDTH 80
#define DEBUG 1
#define LOAD_ONLY 0
#define SUCCESS 0
#define FAIL 1
#define YES 0
#define NONO 1

typedef enum boolean {
	TRUE,
	FALSE
} BOOLEAN;

/* Backbone atoms of an amino acid */
typedef enum pe_atom_name {
	PE_NN,		/* the nitrogen atom of the amino group */
	PE_CA,		/* the central carbon atom */
	PE_CC,		/* the carbon atom of the carboxyl group */
	PE_RNA,		/* RNA or DNA */
	PE_UNKNOWN	/* unknown */
} PE_ATOM_NAME;

/**
 * struct pe_model - is the backbone sequences of a structure model
 * @seq_head: head of the first backbone sequence
 * @seq_tail: tail of the last backbone sequence
 */
struct pe_model {
	int seq_head;
	int seq_tail;
};

/**
 * struct pe_molecule - is the backbones of a molecule
 * @chain_head: head of the first backbone sequence
 * @chain_tail: tail of the last backbone sequence
 * @id        : molecule identifier
 * @name[LINE_WIDTH + 1]: molecule name
 */
struct pe_molecule {
	int chain_head;
	int chain_tail;
	int id;			/* COMPND (1, 2, ... ) */
	char name[LINE_WIDTH + 1];
};

/**
 * struct pe_chain - is the backbone of a polypeptide chain
 * @atom_head: head of the backbone sequence
 * @atom_tail: tail of the backbone sequence
 * @tile_head: head of the corresponding tile sequence
 * @tile_tail: tail of the corresponding tile sequence
 * @id       : chain identifier
 * @mol_id   : molecule identifier
 */
struct pe_chain {
	int atom_head;
	int atom_tail;
	int tile_head;
	int tile_tail;
	char id;		/* COMPND (A, B, ...) */
	int mol_id;
	int mod_num;
};

/**
 * struct pe_atom - is an atom data
 * @serial_num : atom serial number
 * @atom_name  : atom name
 * @alt_lo     : alternate location indicator
 * @res_name[3]: residue name
 * @chain_id   : chain identifier
 * @res_seq    : residue sequence identifier
 * @x          : orthogonal coordinates for x in angstroms
 * @y          : orthogonal coordinates for y in angstroms
 * @z          : orthogonal coordinates for z in angstroms
 * @occupancy  : occupancy
 * @seg_id[4]  : segment identifier
 * @err        : error of the approximation, i.e., |tile pos. - atom pos.|
 * @can_be_init_atom: work flag
 */
struct pe_atom {
	int serial_num;		/* column 7-11 */
	PE_ATOM_NAME atom_name;	/* column 13-16: CA, C, N, ... */
	char alt_loc;		/* column 17 */
	char res_name[3];	/* column 18-20 */
	char chain_id;		/* column 22 */
	int res_seq;		/* column 23-26 */
	float x;		/* coloum 31-38  Real(8.3) */
	float y;		/* coloum 39-46 */
	float z;		/* coloum 47-54 */
	float occupancy;	/* coloum 55-60  Real(6.2) */
	char seg_id[4];		/* coloum 73-76 */
	float err;
	char can_be_init_atom;
};

#define PDB_MOLECULE_MAX 99
#define PDB_CHAIN_MAX (98+1)	/* Note that chain id starts from 1 */
/**
 * struct pe_protein - is a protein data
 * @mod_num   : count of models of the protein
 * @model     : pointer to the models
 * @mol_num   : count of molecules of the protein
 * @molecule[99 + 1]: array of the molecules
 * @chn_num   : count of chains of the protein
 * @chain[(1 + 98) + 1]: array the chains (chain id starts from 1)
 * @res_num   : count of residues of the protein
 * @atm_num   : count of backbone atoms read (#Ca+#N+#C' or #Ca if Ca only)
 * @atom      : pointer to the atoms
 * @seq_num   : count of all atom sequences (= chn_num * mod_num)
 * @sequence  : pointer to the sequences
 * @tile_max_num: The size of the tile array allocated 
 * @tile_num  : count of tiles
 *
 * @best_ind  :
 * @ave_bond_length:
 * @ave_dis   :
 * @ave_error :
 */
struct pe_protein {
	int mod_num;
	struct pe_model * model;
	int mol_num;			/* up to 99 (1, 2, ..., 99) */
	struct pe_molecule molecule[PDB_MOLECULE_MAX + 1];
	int chn_num;			/* up to 98 (A, B, ..., Z, ...) */
	struct pe_chain chain[PDB_CHAIN_MAX + 1];
	int res_num;
	int atm_num;
	struct pe_atom * atom;
	int seq_num;
	struct pe_chain * sequence;
	int tile_max_num;
	int tile_num;

	int best_ind;
	float ave_bond_length;
	float ave_dis;
	float ave_error;
};
